/*
	Copyright (C) 2013 Lukasz Nidecki SQ5RWU

    This file is part of ArduinoQAPRS.

    ArduinoQAPRS is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    ArduinoQAPRS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ArduinoQAPRS; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    Ten plik jest czďż˝ciďż˝ ArduinoQAPRS.

    ArduinoQAPRS jest wolnym oprogramowaniem; moďż˝esz go rozprowadzaďż˝ dalej
    i/lub modyfikowaďż˝ na warunkach Powszechnej Licencji Publicznej GNU,
    wydanej przez Fundacjďż˝ Wolnego Oprogramowania - wedďż˝ug wersji 2 tej
    Licencji lub (wedďż˝ug twojego wyboru) ktďż˝rejďż˝ z pďż˝niejszych wersji.

    Niniejszy program rozpowszechniany jest z nadziejďż˝, iďż˝ bďż˝dzie on
    uďż˝yteczny - jednak BEZ JAKIEJKOLWIEK GWARANCJI, nawet domyďż˝lnej
    gwarancji PRZYDATNOďż˝CI HANDLOWEJ albo PRZYDATNOďż˝CI DO OKREďż˝LONYCH
    ZASTOSOWAďż˝. W celu uzyskania bliďż˝szych informacji siďż˝gnij do
    Powszechnej Licencji Publicznej GNU.

    Z pewnoďż˝ciďż˝ wraz z niniejszym programem otrzymaďż˝eďż˝ teďż˝ egzemplarz
    Powszechnej Licencji Publicznej GNU (GNU General Public License);
    jeďż˝li nie - napisz do Free Software Foundation, Inc., 59 Temple
    Place, Fifth Floor, Boston, MA  02110-1301  USA

 */

/**
 * @file
 * @brief Nagłówki dla klasy QAPRSBase
 */

#ifndef QAPRSBASE_H_
#define QAPRSBASE_H_

#include <Arduino.h>
#include "QAPRSCommon.h"

/**
 * @brief Klasa bazowa do nadawania APRS
 * @details Klasa słuzy jako baza do podimplemntacji generowania AFSK.
 */
class QAPRSBase {
private:
	/**
	 * @brief suma kontrolna pakietu
	 */
	uint16_t ax25CRC;
	/**
	 * @brief ilosć bajtów synchronizacyjnych do nadania przed zawartoscią pakietu
	 */
	static const uint8_t ax25HeaderFlagFieldCount = 100;
	/**
	 * @brief Flaga
	 * @details <http://www.tapr.org/pub_ax25.html#2.2.1>
	 */
	static const uint8_t ax25FlagFieldValue = 0x7E;
	/**
	 * @brief Czas wysyłania podedynczego tonu. W ms.
	 * @details Czas calkowity powinien wynosic 833ms. Wartosć podana tutaj uwzględnia zwłokę związaną z wywoływaniem
	 * funkcji itp.
	 */
	static const uint16_t toneSendTime = 815;
	/**
	 * @brief Czas oczekiwania na zwolnienie kanału.
	 * @details Co 100ms sprawdzamy czy można już nadawać @see canTransmit
	 */
	static const uint16_t channelFreeWaitingMS = 2000; // 2000 ms
	/**
	 * @brief Domylslny czas pomiędzy włączeniem nadawania a rozpoczęciem generowania AFSK
	 */
	static const uint16_t defaultTxDelay = 300; // 300 ms
	/**
	 * @brief Pin Arduino na którym ustawiamy logiczną 1 w momencie nadawania
	 */
	uint8_t txEnablePin;
	/**
	 * @brief Bufor tymczasowy
	 */
	uint8_t tmpData[255];
	/**
	 * @brief Opóźnienie w ms pomiędzy ustawieniem stanu wysokiego na wyjsciu txEnablePin a rozpoczęciem generowania AFSK
	 */
	uint16_t txDelay;

	virtual uint8_t canTransmit();
	void ax25SendHeaderBegin();
	void ax25SendByte(uint8_t byte);
	void ax25SendFooter();
	void ax25CalcCRC(uint8_t ls_bit);
protected:
	/**
	 * @brief Pin Arduino [we] który musi być w stanie niskim aby rozpocząć nadawanie.
	 */
	uint8_t sensePin;
	/**
	 * @brief Obecnie generowany ton
	 */
	QAPRSSendingTone currentTone;

	virtual void initializeRadio();
	virtual void enableTransmission();
	virtual void disableTranssmision();

	virtual void toggleTone();
	virtual void initializeTimer1();
	void delayuSeconds(uint16_t us);
public:
	QAPRSBase() {};
	QAPRSReturnCode send(char * buffer);
	QAPRSReturnCode send(char * buffer, size_t length);
	QAPRSReturnCode send(char * from_addr, uint8_t from_ssid, char * to_addr, uint8_t to_ssid, char * packet_content);
	QAPRSReturnCode send(char * from_addr, uint8_t from_ssid, char * to_addr, uint8_t to_ssid, char * relays, char * packet_content);

	virtual void init(uint8_t sensePin, uint8_t txEnablePin);
	void setTxDelay(uint16_t txDelay);
	virtual void timerInterruptHandler() {};
};

/**
 * @brief Przesuń bajt o 1 bit w lewo. Używane w nagłówku ax.25
 */
#define SHIFT_BYTE(x) (x << 1)

#endif /* QAPRSBASE_H_ */
